# regular imports for general Maya functions Open Maya for extended functions
import maya.cmds as cmds

# GUI related imports regular maya gui is a nightmare to code
from PySide import QtCore, QtGui
from shiboken import wrapInstance
import maya.OpenMayaUI as omui

import copy
import sys

import MLEUtils.AmDataModel as model


LarianShelfName = "AM"
LarianShelfIcon = "AM.png"
LarianshelfTooltip = "Animation Manager for timeline data"


# Global variables
dummyList = []
dataText = ""
animationList = model.AnimationList()

dataCopy = []

launchBool = True


class _TableColumn(object):
    def __init__(self, attributeName, headerTitle):
        self.attributeName = attributeName
        self.headerTitle = headerTitle


_TABLE_COLUMNS = [
    _TableColumn("name", "Name"),
    _TableColumn("start", "Start"),
    _TableColumn("end", "End"),
    _TableColumn("is_bank", "PoseBank")
]

# Global methods


# get maya main window to use as script ui parent
def _getMayaMainWindow():
    main_window_ptr = omui.MQtUtil.mainWindow()
    return wrapInstance(long(main_window_ptr), QtGui.QWidget)  # noqa


def _setTableEntry(index, entry):
    item = QtGui.QTableWidgetItem(entry.name)
    item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
    AMWin.ui.tblAnimData.setItem(index, 0, item)

    item = QtGui.QTableWidgetItem(str(entry.start))
    item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
    AMWin.ui.tblAnimData.setItem(index, 1, item)

    item = QtGui.QTableWidgetItem(str(entry.end))
    item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
    AMWin.ui.tblAnimData.setItem(index, 2, item)

    item = QtGui.QTableWidgetItem("Yes" if entry.is_bank else "No")
    item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
    AMWin.ui.tblAnimData.setItem(index, 3, item)


def _resetTableContents():
    entryCount = len(animationList.entries)

    AMWin.ui.tblAnimData.setRowCount(entryCount)

    for i, entry in enumerate(animationList.entries):
        _setTableEntry(i, entry)

    AMWin.ui.tblAnimData.horizontalHeader().resizeSections()


def _getCurrentDummyRef():
    currIdx = AMWin.ui.cbxDummyList.currentIndex()
    dummyRef = dummyList[currIdx]

    return dummyRef


def _saveAnimationList():
    dummyRef = _getCurrentDummyRef()
    cmds.select(dummyRef)
    model.set_animation_list(dummyRef, animationList)


# Edit a Data Entry
def editAmEntry(entryIndex, attributeName, value):
    entry = animationList.entries[entryIndex]

    assert hasattr(entry, attributeName)

    setattr(entry, attributeName, value)

    _saveAnimationList()
    _setTableEntry(entryIndex, entry)


def toggleBoolValue(entryIndex, column):
    entry = animationList.entries[entryIndex]
    attributeName = _TABLE_COLUMNS[column].attributeName

    value = getattr(entry, attributeName)

    editAmEntry(entryIndex, attributeName, not value)


# Delete a Data Entry
def deleteAmEntries(selectionList):
    global animationList

    oldEntryCount = len(animationList.entries)

    newEntries = [animationList.entries[i] for i in range(oldEntryCount) if i not in selectionList]

    animationList.entries = newEntries

    _saveAnimationList()
    _resetTableContents()


# Fill Ui Table
def setTableData(refObject):
    global animationList

    # get all the am data if it exists and show it in the table
    if not model.has_animation_list(refObject):
        return

    animationList = model.get_animation_list(refObject)

    _resetTableContents()


# Entry creation method
def addAmEntry(entry):
    animationList.entries.append(entry)

    entryCount = len(animationList.entries)
    AMWin.ui.tblAnimData.setRowCount(entryCount)

    _saveAnimationList()
    _setTableEntry(entryCount - 1, entry)


# Create main UI
class AnimationManagerUI(object):
    def setupUi(self, Dialog):
        Dialog.resize(497, 531)  # size

        self.vLayoutMain = QtGui.QVBoxLayout(Dialog)

        self.lblRoot = QtGui.QLabel(Dialog)
        self.lblRoot.setText("Pick your RootDummy")
        self.vLayoutMain.addWidget(self.lblRoot)

        self.cbxDummyList = QtGui.QComboBox(Dialog)
        self.vLayoutMain.addWidget(self.cbxDummyList)

        self.hLayoutTable = QtGui.QHBoxLayout()
        self.hLayoutTable.setContentsMargins(10, -1, 10, -1)

        self.tblAnimData = QtGui.QTableWidget(Dialog)
        self.tblAnimData.setRowCount(0)

        self.tblAnimData.setColumnCount(len(_TABLE_COLUMNS))

        header = self.tblAnimData.horizontalHeader()

        for i, column in enumerate(_TABLE_COLUMNS):
            item = QtGui.QTableWidgetItem()
            item.setText(column.headerTitle)
            self.tblAnimData.setHorizontalHeaderItem(i, item)

            if i == 0:
                resize_mode = QtGui.QHeaderView.Stretch
            else:
                resize_mode = QtGui.QHeaderView.ResizeToContents

            header.setResizeMode(i, resize_mode)

        header.resizeSections()

        self.tblAnimData.verticalHeader().setVisible(False)
        self.tblAnimData.verticalHeader().setDefaultSectionSize(19)
        self.tblAnimData.verticalHeader().setMinimumSectionSize(10)

        self.hLayoutTable.addWidget(self.tblAnimData)
        self.vLayoutMain.addLayout(self.hLayoutTable)

        self.FrmBtnContainer = QtGui.QFrame(Dialog)
        self.FrmBtnContainer.setMinimumSize(QtCore.QSize(350, 25))
        self.FrmBtnContainer.setMaximumSize(QtCore.QSize(350, 25))

        self.horizontalLayout = QtGui.QWidget(self.FrmBtnContainer)
        self.horizontalLayout.setGeometry(QtCore.QRect(0, 0, 351, 26))

        self.mainBtnContainer = QtGui.QHBoxLayout(self.horizontalLayout)
        self.mainBtnContainer.setSpacing(0)
        self.mainBtnContainer.setContentsMargins(0, 0, 0, 0)

        self.frmLeftBtnContainer = QtGui.QFrame(self.horizontalLayout)
        self.frmLeftBtnContainer.setMinimumSize(QtCore.QSize(140, 24))
        self.frmLeftBtnContainer.setMaximumSize(QtCore.QSize(140, 24))

        self.btnNew = QtGui.QPushButton(self.frmLeftBtnContainer)
        self.btnNew.setGeometry(QtCore.QRect(10, 0, 41, 23))
        self.btnNew.setMaximumSize(QtCore.QSize(41, 23))
        self.btnNew.setText("New")

        self.btnDelete = QtGui.QPushButton(self.frmLeftBtnContainer)
        self.btnDelete.setGeometry(QtCore.QRect(70, 0, 41, 23))
        self.btnDelete.setMaximumSize(QtCore.QSize(41, 23))
        self.btnDelete.setText("Delete")

        self.mainBtnContainer.addWidget(self.frmLeftBtnContainer)

        self.btnSelectAll = QtGui.QPushButton(self.horizontalLayout)
        self.btnSelectAll.setMaximumSize(QtCore.QSize(75, 23))
        self.btnSelectAll.setText("Select All")

        self.mainBtnContainer.addWidget(self.btnSelectAll)

        self.frmRightBtnContainer = QtGui.QFrame(self.horizontalLayout)
        self.frmRightBtnContainer.setMinimumSize(QtCore.QSize(150, 24))
        self.frmRightBtnContainer.setMaximumSize(QtCore.QSize(150, 24))

        self.btnCopy = QtGui.QPushButton(self.frmRightBtnContainer)
        self.btnCopy.setGeometry(QtCore.QRect(20, 0, 41, 23))
        self.btnCopy.setMaximumSize(QtCore.QSize(41, 23))
        self.btnCopy.setText("Copy")

        self.btnPaste = QtGui.QPushButton(self.frmRightBtnContainer)
        self.btnPaste.setGeometry(QtCore.QRect(80, 0, 51, 23))
        self.btnPaste.setMaximumSize(QtCore.QSize(51, 23))
        self.btnPaste.setText("Paste")

        self.mainBtnContainer.addWidget(self.frmRightBtnContainer)
        self.vLayoutMain.addWidget(self.FrmBtnContainer)

        self.lblTooltip = QtGui.QLabel(Dialog)
        self.lblTooltip.setText("Alt+Click on column sets the viewport timerange.")
        self.vLayoutMain.addWidget(self.lblTooltip)


# Create additemUI
class AddAmEntryUi(object):
    def setupUi(self, Dialog):
        Dialog.resize(297, 102)
        Dialog.setMinimumSize(QtCore.QSize(0, 102))
        Dialog.setMaximumSize(QtCore.QSize(16777215, 102))

        self.vLayMain = QtGui.QVBoxLayout(Dialog)
        self.VLaySub = QtGui.QVBoxLayout()
        self.HLayTop = QtGui.QHBoxLayout()

        self.lblName = QtGui.QLabel(Dialog)
        self.lblName.setText("Name")
        self.HLayTop.addWidget(self.lblName)

        self.txtName = QtGui.QLineEdit(Dialog)
        self.HLayTop.addWidget(self.txtName)

        validator = QtGui.QRegExpValidator(QtCore.QRegExp("([a-z0-9A-Z_])*"), self.txtName)
        self.txtName.setValidator(validator)

        self.btnAddFName = QtGui.QPushButton(Dialog)
        self.btnAddFName.setMinimumSize(QtCore.QSize(91, 0))
        self.btnAddFName.setMaximumSize(QtCore.QSize(91, 16777215))
        self.btnAddFName.setText("Add filename")
        self.HLayTop.addWidget(self.btnAddFName)

        self.VLaySub.addLayout(self.HLayTop)

        self.HLayBottom = QtGui.QHBoxLayout()

        self.lblBegin = QtGui.QLabel(Dialog)
        self.lblBegin.setMaximumSize(QtCore.QSize(46, 16777215))
        self.lblBegin.setText("Begin")
        self.HLayBottom.addWidget(self.lblBegin)

        self.sbxBegin = QtGui.QSpinBox(Dialog)
        self.sbxBegin.setMaximum(sys.maxint)
        self.sbxBegin.setMinimum(-sys.maxint-1)
        self.HLayBottom.addWidget(self.sbxBegin)

        self.lblEnd = QtGui.QLabel(Dialog)
        self.lblEnd.setMaximumSize(QtCore.QSize(43, 16777215))
        self.lblEnd.setText("End")
        self.HLayBottom.addWidget(self.lblEnd)

        self.sbxEnd = QtGui.QSpinBox(Dialog)
        self.sbxEnd.setMaximum(sys.maxint)
        self.sbxEnd.setMinimum(-sys.maxint-1)
        self.HLayBottom.addWidget(self.sbxEnd)

        self.btnTimeRange = QtGui.QPushButton(Dialog)
        self.btnTimeRange.setMinimumSize(QtCore.QSize(91, 0))
        self.btnTimeRange.setMaximumSize(QtCore.QSize(91, 16777215))
        self.btnTimeRange.setText("Timeline range")
        self.HLayBottom.addWidget(self.btnTimeRange)

        self.VLaySub.addLayout(self.HLayBottom)
        self.vLayMain.addLayout(self.VLaySub)

        self.btnInsertNewEntry = QtGui.QPushButton(Dialog)
        self.btnInsertNewEntry.setText("Insert New Animation")
        self.vLayMain.addWidget(self.btnInsertNewEntry)


# Create editNameUI
class EditEntryNameUi(object):
    def setupUi(self, Dialog):
        Dialog.resize(297, 70)
        Dialog.setMinimumSize(QtCore.QSize(0, 70))
        Dialog.setMaximumSize(QtCore.QSize(16777215, 70))

        self.vLayoutMain = QtGui.QVBoxLayout(Dialog)

        self.hLayout = QtGui.QHBoxLayout()

        self.lblName = QtGui.QLabel(Dialog)
        self.lblName.setText("Name")
        self.hLayout.addWidget(self.lblName)

        self.txtName = QtGui.QLineEdit(Dialog)
        self.txtName.setGeometry(QtCore.QRect(60, 10, 131, 20))
        self.hLayout.addWidget(self.txtName)

        validator = QtGui.QRegExpValidator(QtCore.QRegExp("([a-z0-9A-Z_])*"), self.txtName)
        self.txtName.setValidator(validator)

        self.btnAddFName = QtGui.QPushButton(Dialog)
        self.btnAddFName.setMinimumSize(QtCore.QSize(91, 0))
        self.btnAddFName.setText("Add filename")

        self.hLayout.addWidget(self.btnAddFName)
        self.vLayoutMain.addLayout(self.hLayout)

        self.btnEditEntry = QtGui.QPushButton(Dialog)
        self.btnEditEntry.setText("Edit animation name")
        self.vLayoutMain.addWidget(self.btnEditEntry)


# Create EditTimeUI
class EditEntryTimeUi(object):
    def setupUi(self, Dialog):
        Dialog.resize(257, 72)

        self.btnTimeRange = QtGui.QPushButton(Dialog)
        self.btnTimeRange.setGeometry(QtCore.QRect(150, 10, 91, 23))
        self.btnTimeRange.setText("Timeline range")

        self.btnEditEntry = QtGui.QPushButton(Dialog)
        self.btnEditEntry.setGeometry(QtCore.QRect(10, 40, 240, 23))
        self.btnEditEntry.setText("Edit animation time")

        self.lblBegin = QtGui.QLabel(Dialog)
        self.lblBegin.setGeometry(QtCore.QRect(20, 13, 46, 13))
        self.lblBegin.setText("Time")

        self.sbxBegin = QtGui.QSpinBox(Dialog)
        self.sbxBegin.setGeometry(QtCore.QRect(60, 10, 80, 22))
        self.sbxBegin.setMaximum(sys.maxint)
        self.sbxBegin.setMinimum(-sys.maxint-1)


# Create Insert Window
class AddAmEntryWindow(QtGui.QDialog):
    def __init__(self, parent=None):

        super(AddAmEntryWindow, self).__init__(parent)
        self.setWindowFlags(QtCore.Qt.Tool)

        self.ui = AddAmEntryUi()
        self.ui.setupUi(self)
        self.setWindowTitle("Insert New Animation")

        self.ui.btnAddFName.clicked.connect(self.autoSetName)
        self.ui.btnTimeRange.clicked.connect(self.autoSetRange)

        self.ui.btnInsertNewEntry.clicked.connect(self.addEntry)

        self.activateWindow()

        self.ui.txtName.setFocus()

    def autoSetName(self):
        fileName = cmds.file(q=True, shortName=True, sceneName=True)
        self.ui.txtName.setText(fileName[:len(fileName)-3])
        # this fancy notation is called slicing [start:end:step]

    def autoSetRange(self):
        startTime = cmds.playbackOptions(query=True, minTime=True)
        endTime = cmds.playbackOptions(query=True, maxTime=True)

        self.ui.sbxBegin.setValue(startTime)
        self.ui.sbxEnd.setValue(endTime)

    def addEntry(self):
        newName = self.ui.txtName.text()
        newName = newName.replace(" ", "")

        newEntry = model.AnimationEntry(newName, self.ui.sbxBegin.value(), self.ui.sbxEnd.value(), False)

        addAmEntry(newEntry)
        self.close()


# Create edit Name window
class EditEntryNameWindow(QtGui.QDialog):
    def __init__(self, parent=None, row=-1, column=-1):
        super(EditEntryNameWindow, self).__init__(parent)
        self.setWindowFlags(QtCore.Qt.Tool)

        self.ui = EditEntryNameUi()
        self.ui.setupUi(self)
        self.setWindowTitle("Edit Name")

        self.ui.btnAddFName.clicked.connect(self.autoSetName)

        self.ui.btnEditEntry.clicked.connect(self.editEntry)

        self.ui.txtName.returnPressed.connect(self.editEntry)

        self.entryIndex = row
        self.attributeName = _TABLE_COLUMNS[column].attributeName

        self.activateWindow()

        entry = animationList.entries[self.entryIndex]
        value = getattr(entry, self.attributeName)

        self.ui.txtName.setText(value)

        self.ui.txtName.setFocus()

    def autoSetName(self):
        fileName = cmds.file(q=True, shortName=True, sceneName=True)
        self.ui.txtName.setText(fileName[:len(fileName)-3])
        # this fancy notation is called slicing [start:end:step]

    def editEntry(self):
        newName = self.ui.txtName.text()

        newName = newName.replace(" ", "")

        editAmEntry(self.entryIndex, self.attributeName, newName)
        self.close()


# Create edit Time Window
class EditEntryTimeWindow(QtGui.QDialog):
    def __init__(self, parent=None, row=-1, column=-1):
        super(EditEntryTimeWindow, self).__init__(parent)
        self.setWindowFlags(QtCore.Qt.Tool)

        self.ui = EditEntryTimeUi()
        self.ui.setupUi(self)
        self.setWindowTitle("Edit Time")

        self.ui.btnTimeRange.clicked.connect(self.autoSetRange)

        self.ui.btnEditEntry.clicked.connect(self.editEntry)

        self.attributeName = _TABLE_COLUMNS[column].attributeName
        self.entryIndex = row

        entry = animationList.entries[self.entryIndex]
        value = int(getattr(entry, self.attributeName))

        self.activateWindow()
        self.ui.sbxBegin.setValue(value)
        self.ui.sbxBegin.setFocus()

        self.r = row
        self.c = column

    def autoSetRange(self):
        if self.c == 1:
            startTime = cmds.playbackOptions(query=True, minTime=True)
            self.ui.sbxBegin.setValue(startTime)
        else:
            endTime = cmds.playbackOptions(query=True, maxTime=True)
            self.ui.sbxBegin.setValue(endTime)

    def editEntry(self):
        editAmEntry(self.entryIndex, self.attributeName, self.ui.sbxBegin.value())
        self.close()


# Create main Window
class AMWindow(QtGui.QDialog):
    def __init__(self, parent=None):
        super(AMWindow, self).__init__(parent)
        self.setWindowFlags(QtCore.Qt.Tool)

        self.ui = AnimationManagerUI()
        self.ui.setupUi(self)
        self.setWindowTitle("Animation Manager")

        self.ui.cbxDummyList.currentIndexChanged.connect(self.dummyIndexChanged)

        self.ui.btnNew.clicked.connect(self.newEntryFunc)
        self.ui.btnDelete.clicked.connect(self.deleteEntryFunc)

        self.ui.btnSelectAll.clicked.connect(self.selectEntriesFunc)

        self.ui.btnCopy.clicked.connect(self.copyEntryFunc)
        self.ui.btnPaste.clicked.connect(self.pasteEntryFunc)

        self.ui.tblAnimData.cellDoubleClicked.connect(self.editCellContent)

        self.ui.tblAnimData.cellClicked.connect(self.setViewportTimeRange)

    # new button function
    def newEntryFunc(self):
        insertWin = AddAmEntryWindow(parent=_getMayaMainWindow())
        insertWin.show()

    def setViewportTimeRange(self, row, column):
        modifiers = QtGui.QApplication.keyboardModifiers()

        if modifiers == QtCore.Qt.AltModifier:
            entry = animationList.entries[row]
            cmds.playbackOptions(minTime=entry.start, maxTime=entry.end)

    def dummyIndexChanged(self, index):
        setTableData(dummyList[index])

    # delete button function
    def deleteEntryFunc(self):
        if len(self.ui.tblAnimData.selectedIndexes()) > 0:
            rowSelect = []

            for idx in self.ui.tblAnimData.selectedIndexes():
                rowSelect.append(idx.row())

            deleteAmEntries(rowSelect)

    def selectEntriesFunc(self):
        self.ui.tblAnimData.selectColumn(0)

    def copyEntryFunc(self):
        dataCopy[:] = []

        if len(self.ui.tblAnimData.selectedIndexes()) > 0:
            for idx in self.ui.tblAnimData.selectedIndexes():
                dataCopy.append(idx.row())

    def pasteEntryFunc(self):
        for i in dataCopy:
            entryCopy = copy.deepcopy(animationList.entries[i])

            addAmEntry(entryCopy)

    # event handler for window open takes object and event as vars
    def showEvent(self, e):
        layers = cmds.ls(long=True, type='displayLayer')

        sceneObjects = cmds.ls()

        for obj in sceneObjects:
            if cmds.objectType(obj) == 'joint':
                if 'Dummy_Root' in obj:
                    dummyList.append(obj)
                for layer in layers:
                    if layer in obj and 'Dummy_' in obj and '|' not in obj:
                        dummyList.append(obj)

        if dummyList is None or len(dummyList) == 0:
            global launchBool
            launchBool = False
        else:
            self.ui.cbxDummyList.clear()
            self.ui.cbxDummyList.addItems(dummyList)

            setTableData(dummyList[0])

    def editCellContent(self, row, column):
        if column == 0:
            editNameWin = EditEntryNameWindow(parent=_getMayaMainWindow(), row=row, column=column)
            editNameWin.show()
        elif column == 3:
            toggleBoolValue(row, column)
        else:
            editTimeWin = EditEntryTimeWindow(parent=_getMayaMainWindow(), row=row, column=column)
            editTimeWin.show()


# start up sequence


reload(model)  # noqa


# check if window exists already by trying to close it ignore any errors
try:
    AMWin.close()  # noqa
except:
    pass

AMWin = AMWindow(parent=_getMayaMainWindow())
AMWin.show()

if not launchBool:
    QtGui.QMessageBox.warning(_getMayaMainWindow(),
                              "Error",
                              "Your current scene has no Dummy_Root! Animation Manager won't run",
                              QtGui.QMessageBox.Ok)
    AMWin.close()
