LarianShelfName = "MLE"
LarianShelfIcon = "MLE.png"
LarianshelfTooltip = "Layer exporter for Larian projects"

#regular imports for general Maya functions Open Maya for extended functions
import maya.cmds as cmds
import maya.api.OpenMaya as omCmds

#GUI related imports regular maya gui is a nightmare to code
from PySide import QtCore, QtGui
from shiboken import wrapInstance
import maya.OpenMayaUI as omui
import maya.mel as mel

#bullet stuff
import maya.app.mayabullet.BulletUtils as BulletUtils
import maya.app.mayabullet.RigidBody as RigidBody

#python specific imports
from itertools import ifilter
import re
import sys
import subprocess
import os

#LarianUtil import
mainPath = os.environ['MAYA_APP_DIR']+ "/scripts/LarianStudiosModTools/" #Best guess for maya path install
UiPath = "QtUi/"

sys.path.append(mainPath+UiPath)
sys.path.append(mainPath) #add the path for the util package
from MLEUtils import MLEUtils, GrannyAnimationExporter, GrannyMeshExporter, BulletPhysExporter
import MLEUtils.AmDataModel as model
from QtUi import MLEUi

#get maya main window to use as script ui parent
def maya_main_window():
    main_window_ptr = omui.MQtUtil.mainWindow()
    return wrapInstance(long(main_window_ptr), QtGui.QWidget)
    
#Global Variables
global dummyRoot

#DataLists
fullLayerList = []
fullAnimList = []
exportDirList = []
exportHistoryList = []

#File help
granSetMesh = "Gr2_Presets\Mesh_ExportSettings_Maya.ges"
granSetVertCols = "Gr2_Presets\Mesh_VertexClrs_ExportSettings_Maya.ges"
granSetAnim = "Gr2_Presets\Animation_Export_Settings_Maya.ges"
currentPath = ""
targetPath = "" #used for actualExport

defaultScriptPath=""#used to save history

GameDataPath=""#game data placeholder path


#ExportRegEx
physCheck =  re.compile(".*_PHYS",re.IGNORECASE)
boneCheck =  re.compile(".*_BONE.*|.*BONE_.*",re.IGNORECASE)
nonConstrBoneCheck = re.compile(".*_BONE$|.*BONE_.*",re.IGNORECASE)
groupCheck = re.compile(".*GRP.*",re.IGNORECASE)

endScript = False

#batch variables
mayaPath = '' #mayapy placeholder path
meshScriptPath = 'StandaloneScripts/BatchGrannyMeshExporter.py'
loopScriptPath = 'StandaloneScripts/FileLooper.py'
animScriptPath = 'StandaloneScripts/BatchGrannyAnimationExporter.py'


#main export method takes care of all the things
def exportMain(doEditSettings, doIncludeBones, doVertexColor, doCreateDebugLog, doExpressExport, doVisual, doPhysics, doAnimation, selectedLayers, selectedAnimEntries, animExportQuality):       
    
    print 'started'
    global mayaPath
    
    if doCreateDebugLog:
        MLEUtils.startDebugLog()

    #make sure the granny plugin is loaded
    try:
        cmds.loadPlugin("larian_maya2015_x64.mll")
    except:
        QtGui.QMessageBox.warning(maya_main_window(), "Granny plugin error", "Failed to load the granny plugin. Aborted export. Use The Divinity Engine installer to install granny plugin",QtGui.QMessageBox.Ok)
        return
        
    #check if target path exists and cancel export if it doesn't
    if not os.path.exists(targetPath):
        QtGui.QMessageBox.warning(maya_main_window(), "Missing Location", "Saving to a not-existing directory!",QtGui.QMessageBox.Ok)
        return
        
    #check if visual presets exists and cancel export if it doesn't
    if doVisual:
        if not os.path.exists(mainPath+granSetMesh) or not os.path.exists(mainPath+granSetVertCols) :
            QtGui.QMessageBox.warning(maya_main_window(), "Granny preset error", "Failed to load the granny presets for meshes. Check if " + mainPath+granSetMesh + " exists" ,QtGui.QMessageBox.Ok)
            return

    #check if Animation presets exists and cancel export if it doesn't
    if doAnimation:
        if not os.path.exists(mainPath+granSetAnim)  :
            QtGui.QMessageBox.warning(maya_main_window(), "Granny preset error", "Failed to load the granny presets for Animations. Check if " + mainPath+granSetAnim + " exists",QtGui.QMessageBox.Ok)
            return
    
    #get the maya install directory to find the mayapy.py file
    paths = os.environ['PATH'].split(';')
    for path in paths:
        search_result = re.search('/Maya2015/bin', path)
        if search_result and not 'Cg' in path:
            mayaPath = path + "/mayapy.exe"
    
    #check if namespaces are empty     
    nameSpaces = cmds.namespaceInfo(listOnlyNamespaces=True) or []
        
    for ns in nameSpaces:
        if len(cmds.ls(ns+":*", type='joint'))>0:
            QtGui.QMessageBox.warning(maya_main_window(), "Namespaces detected", "Found namespaces in current file. Possible matching errors for bones in the divinity engine." ,QtGui.QMessageBox.Ok)
            break

    bonesList = []
    #Animation files can't do most of the following code
    if doVisual:
        
        if not doExpressExport and cmds.file(q=True, anyModified=True):
           QtGui.QMessageBox.warning(maya_main_window(), "Save Warning" ,"File seems to have outstanding changes that won't be applied during batch export. For test mesh exports use Express Export in advanced settings." )
                 
        exportableLayers = []
    
        for lay in selectedLayers:
            print targetPath + lay + ".lsm"
            
            contentTest = cmds.editDisplayLayerMembers(lay, q=True)
            
            if contentTest is None:
                QtGui.QMessageBox.warning(maya_main_window(), "Layer Error" ,"Layer with name " + lay + " found to be empty. Skipping during export." )
                continue                
            if 'Ragdoll' in lay:
                continue    
                   
            exportableLayers.append(lay)
    
        print selectedLayers 
        print exportableLayers     
        
        if doExpressExport:
            GrannyMeshExporter.GrannyMeshExport(cmds.file(q=True, sceneName=True), targetPath, exportableLayers, doVisual, doVertexColor, doIncludeBones, doEditSettings, doExpressExport)
            
        else:
         
            standaloneCommand = mayaPath + ' ' + mainPath+meshScriptPath + ' ' + cmds.file(q=True, sceneName=True) + ' ' + targetPath + ' ' + ','.join(exportableLayers) + ' ' + str(doVisual) + ' ' + str(doVertexColor) + ' ' + str(doPhysics) + ' ' + str(doIncludeBones) + ' ' + str(doEditSettings)
            
            print standaloneCommand
            
            batchExecuter = subprocess.Popen(standaloneCommand, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            
            out, err = batchExecuter.communicate()

            exitcode = batchExecuter.returncode

            if str(exitcode) != '0':
                MLEUtils.writeToDebugLog(err)
                print (out)
                print 'error running script'
            else:
                print (err)
                print (out)
                print 'exported succeeded FROM MAIN'
                
    if doPhysics:
        
        for layer in selectedLayers:       
            BulletPhysExporter.ExportPhys(layer, targetPath + layer + ".bullet")
            
            

    # Do animation all the way at the end it doesn't need to be repeated over and over again per layer it should be its own loop
    # Additionally anim files won't allow layer exports
    # base files will require a file loop
    if doAnimation:
        exportAnim(selectedAnimEntries, doEditSettings, doCreateDebugLog, animExportQuality) 
        
    if doCreateDebugLog:
        MLEUtils.showDebugFile()

       
#the final hurdle for Anim export
def exportAnim(selectedAnimEntries, doEditSettings, doCreateDebugLog, animExportQuality):
    
    animBonesList = []
    sceneContents = cmds.ls()
    
    animVarList = []
    
    exportableAnims = selectedAnimEntries
    
        
    if MLEUtils.isAnimBaseFile():
        if not os.path.exists(targetPath+cmds.file(q=True, sceneName=True, shortName=True)[:-3]):
            os.makedirs(targetPath+cmds.file(q=True, sceneName=True, shortName=True)[:-3])
            exportableAnims = selectedAnimEntries
                    
    print exportableAnims
    
    if len(exportableAnims) <=0:
        return
                
    #BaseFile 0
    if MLEUtils.isAnimBaseFile():
        animVarList.append(cmds.file(q=True, sceneName=True))
    else:
        animVarList.append(cmds.file(q=True, sceneName=True)[:cmds.file(q=True, sceneName=True).rfind('/')]+".ma")
    
    #TargetPath 1
    if MLEUtils.isAnimFile():
        animVarList.append(targetPath[:targetPath[:-1].rfind('/')+1])
    else:
        animVarList.append(targetPath)  
    
    print animScriptPath[1]
    
    #DoFullRange 2
    if MLEUtils.isAnimFile():
        if len(selectedAnimEntries) == 0:
            animVarList.append('True')
        else:
            if selectedAnimEntries[0] == "No Root Dummy found" or selectedAnimEntries[0] == "No Animation Manager Data Found":
                animVarList.append('True')
            else:
                animVarList.append('False')
    else:
        animVarList.append('True')
    
    animVarList.append(str(doEditSettings)) # 3
    
    
    animVarList.append(str(doCreateDebugLog)) # 4
    
    animVarList.append(str(animExportQuality)) #5
    
    print animVarList[2]
    #Range Selection 8
    if animVarList[2] in 'False':
        
        amDataList = []
        for entry in selectedAnimEntries:
            temp = entry.split(' ')
            
            amDataList.append('*'.join(temp))
        
        animVarList.append('' + ','.join(amDataList) + '')
    
    
    
    fileList = []
    
    #AnimFile/filesToLoop 9
    if MLEUtils.isAnimFile():
        fileList.append(cmds.file(q=True, sceneName=True))
    else:
        animationLocation = cmds.file(q=True, sceneName=True)[:-3]+"/"
        
        for file in exportableAnims:
            fileList.append(animationLocation + file)
    
    
    
    print fileList
    
    
    print len(fileList)
    
    file = open(mainPath+"fileList.txt",'w')
    
    file.write('#'.join(fileList))
        
    file.close()
    
    if MLEUtils.isAnimBaseFile():
    
        standaloneCommand = mayaPath + ' ' + mainPath+loopScriptPath + ' ' + mainPath+"fileList.txt" + ' ' + mainPath+animScriptPath + ' ' + '#'.join(animVarList)
           
        print standaloneCommand
        print "ANIM COMMAND HERE"
        
        
        batchExecuter = subprocess.Popen(standaloneCommand,stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            
        out, err = batchExecuter.communicate()

        exitcode = batchExecuter.returncode

        if str(exitcode) != '0':
            print exitcode
            print (err)
            print (out)
            print 'error running script'
        else:
            print (out)
            print (err)
            
            print 'exported succeeded FROM ANIM'
            
        if exitcode == -2:
            QtGui.QMessageBox.warning(maya_main_window(), "Export Warning" ,"Export succeeded. But found namespaces in files. Check script listener for details." )
            
            print '______________________________________________________'
            print '                 ANIM EXPORT WARNING'
            print '______________________________________________________'
            
            print (out)
            
            print '______________________________________________________'
            print '               ^^ANIM EXPORT WARNING^^'
            print '______________________________________________________'        
                
    
    print animVarList        
    
    if MLEUtils.isAnimFile():        
        GrannyAnimationExporter.GrannyAnimationExport(cmds.file(q=True, sceneName=True), animVarList[0], animVarList[1], animVarList[2] in 'True', animVarList[8].replace('*', ' ').split(','), animVarList[3] in 'True', True, animVarList[5] in 'True', False, animExportQuality)
     
    os.remove(mainPath+"fileList.txt")
    

 
#get export Directory based on current directory
def getExportPath():
    
    savePath = "" 
     
    try:
     MLEWin
    except NameError:
        projectIndex = 0  
    else:
        projectIndex = MLEWin.ui.cbxTargetProject.currentIndex()
    
    projectData = exportDirList[projectIndex]
    
    print projectData
    
    dataIndex = currentPath.find(projectData[0])
    
    if dataIndex is -1:
        savePath = GameDataPath + "/Public/" + projectData[0] +"/"
        
    else:  
        endIndex = currentPath.rfind("/")
        targetLoc = currentPath[dataIndex:endIndex+1] 
    
        savePath = projectData[1] + targetLoc 
    
    if isPhysFile():
        savePath = savePath[:-1]
        savePath = savePath[:savePath.rfind('/')+1]
    
    
    return savePath   
  
    
    
#set the export dirs for all the mods
def getExportDirs():
    
    modSearchPath = GameDataPath + "/Public/"
    
    print modSearchPath
    
    directoryContent = os.listdir(modSearchPath)

    exportDirList[:] = []

    print directoryContent

    for path in directoryContent:
        print modSearchPath+path
        if os.path.isdir(modSearchPath+path):
            exportDirList.append([path,modSearchPath])

    
    print exportDirList
        
#get the display layers in the file        
def getLayerList():
    
    dummyRoot = MLEUtils.getRootDummy()
    
    if dummyRoot is None or not MLEUtils.isAnimFile():
        
        layerList = cmds.ls(type="displayLayer")
        
        fullLayerList[:] = []
        
        for layer in layerList:
            #We ignore the default layer and excluded layers
            if "default" not in layer and "exclude" not in layer and "Exclude" not in layer:
                fullLayerList.append(layer)
                
        
        return fullLayerList
    else:
        return [cmds.file(q=True, sceneName=True, shortName=True)[:-3]]#animation files just show the file name
        
#replace layer list with flitered version
def filterLayers(filter=""):
    newList = []
    for layer in fullLayerList:
        if re.match(".*"+filter+".*",layer,re.IGNORECASE):
            newList.append(layer)
            
    return newList

#replace animlist with filtered version
def filterAnims(filter=""):
    newList = []
    for anim in fullAnimList:
        if re.match(".*"+filter+".*",anim,re.IGNORECASE):
            newList.append(anim)
            
    return newList        

#Load in the saved history if able throw an error if it isn't found    
def getHistoryList():
    
    tempHistDir = defaultScriptPath+"/export.path"
    
    exportHistoryList[:] = []
   
    if not os.path.exists(tempHistDir) and not os.path.isfile(tempHistDir):
       QtGui.QMessageBox.warning(maya_main_window(), "History error", "No path history defined! \n Click to continue with empty browse history!",QtGui.QMessageBox.Ok)
       return addToHistoryList()
       
    else: 
        histRead = open(tempHistDir,'r')
    
        for line in histRead:
            if len(line) > 0 and not line is "":
                exportHistoryList.append(line.rstrip())
            
        histRead.close()
        return addToHistoryList()

#Add a custom browsed path to the history
def addToHistoryCustom(customTarget):

     #check if history contains the entry already
    if len(exportHistoryList) is 0:
        exportHistoryList.append(customTarget)
        return 0
    
    for i in xrange(0, len(exportHistoryList)):
        if customTarget == exportHistoryList[i]:
            return i
    
    exportHistoryList.append(customTarget)
    return len(exportHistoryList)-1
    
#Add new entries when the project folder changes
def addToHistoryList():
    currentTarget = getExportPath()
    
    #check if history contains the entry already
    if len(exportHistoryList) is 0:
        exportHistoryList.append(currentTarget)
        return 0
    
    for i in xrange(0, len(exportHistoryList)):
        if currentTarget == exportHistoryList[i]:
            return i
    
    exportHistoryList.append(currentTarget)
    return len(exportHistoryList)-1
 
#When closign MLE write away the history so far    
def writeHistoryList():    
    if not len(exportHistoryList)>0:
        return
     
    tempHistDir = defaultScriptPath+"/export.path"
       
    histRead = open(tempHistDir,'w')
    
    
    
    for entry in exportHistoryList:
        histRead.write(entry+"\n")
    
    histRead.close()     

#Check if this is a physics file
def isPhysFile():
    return "_Phys/" in cmds.file(q=True, sceneName=True)
        

#Create Main Window
class MLEWindow(QtGui.QDialog):
    def __init__(self, parent=None, asAnim=False):

        super(MLEWindow, self).__init__(parent)
        self.setWindowFlags(QtCore.Qt.Tool)
        self.ui = MLEUi.Ui_Dialog()
        
        self.ui.setupUi(self)
        self.setWindowTitle("Custom Exporter")
        
        self.ui.wdgAdvanced.setVisible(False)
        
        self.ui.btnBrowse.clicked.connect(self.browseFunc)
        self.ui.btnDelete.clicked.connect(self.deleteHistFunc)
        self.ui.btnExport.clicked.connect(self.ExportFunc)
        self.ui.btnClear.clicked.connect(self.ClearHistFunc)
        self.ui.btnSelectAllAnim.clicked.connect(self.selectAllAnim)
        self.ui.btnSelectAllLayers.clicked.connect(self.selectAllLayers)
        self.ui.btnAdvanced.clicked.connect(self.toggleAdvanced)
        
        self.ui.ckbAnimation.stateChanged.connect(self.showAnimOptions)
        
        self.ui.txtLayerFilter.textEdited.connect(self.filterLayers)
        self.ui.txtAnimFilter.textEdited.connect(self.filterAnims)
        
        self.ui.wdgAnimations.setHidden(not asAnim)

        
        tempList = []
        for entry in exportDirList:
            tempList.append(entry[0])
        
        self.ui.cbxTargetProject.clear()
        self.ui.cbxTargetProject.addItems(tempList)
        self.ui.cbxTargetProject.currentIndexChanged.connect(self.updateHist)
        
        self.ui.lstLayers.clear()
        self.ui.lstLayers.addItems(getLayerList())
        
        if asAnim:
            self.ui.lstAnimations.clear()
            self.ui.lstAnimations.addItems(MLEUtils.getAnimationList())
        
        self.ui.lstHistory.clear()
        currIndx = getHistoryList()
        self.ui.lstHistory.addItems(exportHistoryList)
        self.ui.lstHistory.setCurrentRow(currIndx)
        
        if isPhysFile():
            self.ui.ckbVisual.setEnabled(False)
            self.ui.ckbPhysics.setChecked(True)
            self.ui.ckbAnimation.setEnabled(False)
            self.ui.ckbVertColor.setEnabled(False)
            self.ui.ckbEditSettings.setEnabled(False)
            self.ui.ckbIncludeBones.setEnabled(False)
            self.ui.ckbDebugMode.setEnabled(False)
            self.ui.ckbExpressMode.setEnabled(False)
            self.ui.ckbExpressMode.setChecked(True)
        else:
            if asAnim:
                self.ui.ckbVisual.setEnabled(False)
                self.ui.ckbPhysics.setEnabled(False)
                self.ui.ckbAnimation.setChecked(True)
                self.ui.ckbVertColor.setEnabled(False)
                self.ui.ckbEditSettings.setEnabled(True)
                self.ui.ckbIncludeBones.setEnabled(False)
                self.ui.ckbDebugMode.setEnabled(False)
                self.ui.ckbExpressMode.setEnabled(False)
                self.ui.ckbExpressMode.setChecked(True)
            else:
                self.ui.ckbVisual.setChecked(True)
                self.ui.ckbPhysics.setChecked(True)
    
    def toggleAdvanced(self):
        self.ui.wdgAdvanced.setVisible(self.ui.btnAdvanced.isChecked())
        
        if not self.ui.btnAdvanced.isChecked():
            self.ui.ckbVertColor.setChecked(False)
            self.ui.ckbEditSettings.setChecked(False)
            self.ui.ckbIncludeBones.setChecked(False)
            self.ui.ckbDebugMode.setChecked(False)
            if not MLEUtils.isAnimFile():
                self.ui.ckbExpressMode.setChecked(False)
    
    def selectAllAnim(self):
        self.ui.lstAnimations.selectAll()
        
    def selectAllLayers(self):
        self.ui.lstLayers.selectAll()
    
    def showAnimOptions(self):
        
        if not MLEUtils.isAnimFile():
        
            showAnim = self.ui.ckbAnimation.isChecked()
            
            self.ui.wdgAnimations.setHidden(not showAnim)
                
            self.ui.lstAnimations.clear()
            self.ui.lstAnimations.addItems(MLEUtils.getAnimationList())
       
    def browseFunc(self):

        try:
            newTargetPath = cmds.fileDialog2(fileMode=3)#file mode 3 allows for only directories
        except: #user Canceled
            return
        
        newIndex = addToHistoryCustom(newTargetPath[0]+"/")
        
        self.ui.lstHistory.clear()
        self.ui.lstHistory.addItems(exportHistoryList)
        
        self.ui.lstHistory.setCurrentRow(newIndex)
        
    def deleteHistFunc(self):
        
        deleteIndex = self.ui.lstHistory.currentRow()
        
        del exportHistoryList[deleteIndex]
        
        self.ui.lstHistory.clear()
        

        newIndex = addToHistoryList() #select currently relevant entry and readd it if it's the deleted entry
              
        self.ui.lstHistory.addItems(exportHistoryList)
        self.ui.lstHistory.setCurrentRow(newIndex)
    
    #Get all the export data and launch the main export function   
    def ExportFunc(self):
        
        global targetPath
        
        targetPath = self.ui.lstHistory.currentItem().text()
        
        selectedLayers = self.ui.lstLayers.selectedItems()
        
        doEditSettings = self.ui.ckbEditSettings.isChecked()
        doIncludeBones = self.ui.ckbIncludeBones.isChecked()
        doVertexColor = self.ui.ckbVertColor.isChecked()
        doCreateDebugLog = self.ui.ckbDebugMode.isChecked()
        doExpressExport = self.ui.ckbExpressMode.isChecked()
        
        
        doVisual = self.ui.ckbVisual.isChecked()
        doPhysics = self.ui.ckbPhysics.isChecked()
        doAnimation = self.ui.ckbAnimation.isChecked()

        if doVisual or doPhysics:
            if len(selectedLayers) > 0:
                for i in xrange(len(selectedLayers)):
                    selectedLayers[i] = selectedLayers[i].text()
            else:
                doVisual = False
                doPhysics = False
        
        selectedAnimEntries= []
        animExportQuality = self.ui.cbxAnimationQuality.currentIndex() 
        
        if doAnimation:
            selectedAnimEntries = self.ui.lstAnimations.selectedItems()
            
            if len(selectedAnimEntries) > 0:
                for i in xrange(len(selectedAnimEntries)):
                    selectedAnimEntries[i] = selectedAnimEntries[i].text()
            else:
                doAnimation = False
        
        
        if doVisual or doPhysics or doAnimation:
            exportMain(doEditSettings,doIncludeBones,doVertexColor, doCreateDebugLog, doExpressExport, doVisual, doPhysics, doAnimation, selectedLayers, selectedAnimEntries, animExportQuality)
        
    def ClearHistFunc(self):
        
        exportHistoryList[:] = []
        
        self.ui.lstHistory.clear()

        newIndex = addToHistoryList() #select currently relevant entry and readd it if it's the deleted entry
              
        self.ui.lstHistory.addItems(exportHistoryList)
        self.ui.lstHistory.setCurrentRow(newIndex)
        
    def filterLayers(self, e):
        
        self.ui.lstLayers.clear()
        self.ui.lstLayers.addItems(filterLayers(filter=self.ui.txtLayerFilter.text()))
        
    def filterAnims(self, e):
        
        self.ui.lstAnimations.clear()
        self.ui.lstAnimations.addItems(filterAnims(filter=self.ui.txtAnimFilter.text()))
    
    def updateHist(self, index):        
        newIndex = addToHistoryList()
        
        self.ui.lstHistory.clear()
        self.ui.lstHistory.addItems(exportHistoryList)
        
        self.ui.lstHistory.setCurrentRow(newIndex)
        
    def closeEvent(self, event):
        writeHistoryList()
        

#start up sequence

#Check if scene is empty
sceneObjects = cmds.ls(geometry=True)
if len(sceneObjects)==0:
    QtGui.QMessageBox.warning(maya_main_window(), "Empty scene", "Your Scene is empty and has nothing to be exported. \nMLE will not run!",QtGui.QMessageBox.Ok)
else:
    #check if window exists already by trying to close it ignore any errors    
            
    
    currentPath = cmds.file(q=True, sceneName=True)

    projectDirectory = cmds.workspace(q=True, rd=True)
    scriptPathList = os.environ['MAYA_SCRIPT_PATH'].split(";")
    #get the path for history saving
    for path in scriptPathList:
        if projectDirectory in path or path in projectDirectory:
            defaultScriptPath = path
    
    
    dataPathFile = defaultScriptPath+"/data.path" 
    
    if not os.path.exists(dataPathFile):
        QtGui.QMessageBox.warning(maya_main_window(), "Empty Data Path", "Your Game Data path is still empty. Run ModP First and set the data path. \nMLE will not run!",QtGui.QMessageBox.Ok)
    else:
        
                  
        #get set data path          
        
        
        dataRead = open(dataPathFile, 'r')
        
        
        
        GameDataPath = dataRead.readline()
        
        dataRead.close()
        
        if not len(GameDataPath) > 0:
            QtGui.QMessageBox.warning(maya_main_window(), "Empty Data Path", "Your Game Data path is still empty. Run ModP First and set the data path. \nMLE will not run!",QtGui.QMessageBox.Ok) 
        else:    
            getExportDirs() 
    
                
            MLEWin = MLEWindow(parent=maya_main_window(),asAnim=MLEUtils.isAnimFile())
            MLEWin.show()  