import maya.cmds


_ATTRIBUTE_NAME = 'AmData'


class AnimationEntry(object):
    def __init__(self, name=None, start=-1, end=-1, is_bank=False):
        self.name = name
        self.start = start
        self.end = end
        self.is_bank = is_bank

    # It would be really nice to use just JSON, but fixing ALL of copy-paste parsers everywhere is a bit too much
    # We will take it one-by-one and eventually, once everything is using this file, we will be able to switch to
    # normal serialization formats
    @property
    def weird_format(self):
        return "name:{name} start:{start} end:{end} is_bank:{is_bank}".format(**self.__dict__)

    @classmethod
    def make_from_weird_format(cls, data):
        result = cls()

        components = data.split(' ')

        for component in components:
            k, v = component.split(':')

            if k == 'name':
                result.name = v
            elif k == 'start':
                result.start = int(v)
            elif k == 'end':
                result.end = int(v)
            elif k == 'is_bank':
                result.is_bank = v == 'True'

        return result

    def __repr__(self):
        return self.weird_format


class AnimationList(object):
    def __init__(self):
        self.entries = []

    @property
    def _weird_format(self):
        return ",".join([x.weird_format for x in self.entries])

    @classmethod
    def _make_from_weird_format(cls, data):
        result = cls()

        entry_data = data.split(',')

        result.entries = [AnimationEntry.make_from_weird_format(x) for x in entry_data if len(x) > 0]

        return result

    def __repr__(self):
        return self._weird_format

    def __contains__(self, external_entry):
        for entry in self.entries:
            if external_entry.name == entry.name:
                return True

        return False


def parse_animation_list(string):
    return AnimationList._make_from_weird_format(string)


def dump_animation_list(animation_list):
    return animation_list._weird_format


def has_animation_list(node_path):
    return maya.cmds.attributeQuery(_ATTRIBUTE_NAME, node=node_path, exists=True)


def get_animation_list(node_path):
    value = maya.cmds.getAttr(node_path + '.' + _ATTRIBUTE_NAME)
    return parse_animation_list(value)


def set_animation_list(node_path, animation_list):
    value = dump_animation_list(animation_list)

    if not has_animation_list(node_path):
        maya.cmds.addAttr(longName=_ATTRIBUTE_NAME, dataType="string", hidden=True)

    maya.cmds.setAttr(node_path + '.' + _ATTRIBUTE_NAME, value, type='string')

def createDefaultEntry():
    return AnimationEntry(maya.cmds.file(q=True, shortName=True, sceneName=True)[:-3], maya.cmds.playbackOptions(query=True, minTime=True), maya.cmds.playbackOptions(query=True, maxTime=True), False)

def regenerateEntryName(dummyRef):
    animationList = AnimationList()
    #Check if the dummy has am data as an attribute
    
    maya.cmds.select(dummyRef)
        
    if not has_animation_list(dummyRef) :            
        newEntry = createDefaultEntry()
        animationList.entries.append(newEntry)
    else:
        animationList = get_animation_list(dummyRef)
        if len(animationList.entries) <= 0:
            newEntry = createDefaultEntry()
            animationList.entries.append(newEntry)
        else:
            for entry in animationList.entries: 
                nameSplitList = entry.name.split('_')
                entry.name = maya.cmds.file(q=True, shortName=True, sceneName=True)[:-3] + "_" + nameSplitList[len(nameSplitList)-1]

    set_animation_list(dummyRef, animationList)    
        
              