import maya.cmds as cmds
import maya.mel as mel

import sys
import re
import os

from . import MLEUtils
from . import AmDataModel as model

mainPath = os.environ['MAYA_APP_DIR']+ "/scripts/LarianStudiosModTools/" #Best guess for maya path install

#settings file 
granSetPath = "Gr2_Presets/"
granSetAnim = ["Animation_Export_Settings_Maya.ges", "Animation_ExportSettings_MidQ_Maya.ges", "Animation_ExportSettings_HighQ_Maya.ges", "Animation_Export_Settings_Maya_ForJitter.ges", "Animation_Export_Settings_Maya_ForFlurries.ges"]
granSetAnimForBanks = "Animation_Export_Settings_ForBanks_Maya.ges"


def GrannyAnimationExport(AnimFile, BaseFile, TargetPath, doFullRange, rangeSelection, doEditSettings, doExpressExport, doCreateDebugLog, animExportQuality):
    if not doExpressExport:
        cmds.file(AnimFile, open=True, force=True) #open the file to export if the script is running in batch mode
    
    cmds.loadPlugin("larian_maya2015_x64.mll") #load the granny plugin
    
    print animExportQuality
    
    baseFileName = BaseFile[BaseFile.rfind('/')+1 :-3]
    
    animTargetFolder = baseFileName + "/"
    
    entryErrors = []
    
    animBonesList = []
    
    #check for namespaces
    nameSpaces = cmds.namespaceInfo(listOnlyNamespaces=True) or []
        
    for ns in nameSpaces:
        if len(cmds.ls(ns+":*", type='joint'))>0:
            entryErrors.append("Found namespaces in " + AnimFile + ". Possible matching errors for bones in glasses.")
            break
    
    #select all dummy root
    layers = cmds.ls(long=True,type='displayLayer')
    rootList = []
    
    sceneObjects = cmds.ls()
    for obj in sceneObjects:
        if cmds.objectType(obj) == 'joint':
            if 'Dummy_Root' in obj:
                rootList.append(obj)
            for layer in layers:
                if layer in obj and 'Dummy_' in obj and not '|' in obj:
                    rootList.append(obj)
                
            
        
    #Make layers invisible
    layers = cmds.ls(long=True,type='displayLayer')
    for lay in layers:
        if lay.find("defaultLayer") == -1: #default layer can't be set to invisible
            cmds.setAttr(lay+".visibility", 0)
                
    for root in rootList:
        saveDir = TargetPath + animTargetFolder   
        print root
        #get root hierarchy and set related layer visible
        linkedLayer = cmds.listConnections(root, type="displayLayer")[0]
        
        cmds.select(root, hierarchy=True)
        animBonesList.append(cmds.ls(selection=True))
        
        if not linkedLayer is None and not linkedLayer is '':
            cmds.setAttr(linkedLayer+".visibility", 1)
            
        animationList = model.AnimationList()
        
        #Get the AM data on the dummy root if possible
        if model.has_animation_list(root):
            animationList = model.get_animation_list(root)
        
        print "debugging"
        print doFullRange
        
        if doFullRange:
            print animationList.entries
            if len(animationList.entries) > 0:
                print animationList.entries
                for entry in animationList.entries:
                    #Export by setting the timeline
                    setAnimRangeForExport(saveDir,entry,animBonesList, doEditSettings, animExportQuality)
        elif len(rangeSelection) == 0:
            if root is None:
                filename = saveDir + cmds.file(q=True, sceneName=True, shortName=True)[:-3]
                exportAnimFile(filename, animBonesList, granSetPath+granSetAnim[animExportQuality], doEditSettings,) 
        else:
            print rangeSelection
            for entry_data in rangeSelection:
                print entry_data
                entry = model.AnimationEntry.make_from_weird_format(entry_data)

                if entry in animationList:
                    setAnimRangeForExport(saveDir, entry, animBonesList, doEditSettings, animExportQuality)
        
        #set related layer invisible
        if not linkedLayer is None and not linkedLayer is '':
            cmds.setAttr(linkedLayer+".visibility", 0) 
    
    if len(entryErrors) > 0:
        output = ''
        for warning in entryErrors:
            output = output + warning + "\n"
        
        
        if doCreateDebugLog:
            MLEUtils.writeToDebugLog(output)

    
    #print "ending the anim"
    #sys.exit(0)           

#set the range of the time line according to given data when available
def setAnimRangeForExport(saveDir, entry, animBonesList, doEditSettings, animExportQuality):
    saveDir += entry.name
   
    start = entry.start
    end = entry.end
    settings = granSetPath+granSetAnim[animExportQuality]
   
    if entry.is_bank:
        start -= 1
        settings = granSetPath+granSetAnimForBanks

    cmds.playbackOptions(minTime=start,maxTime=end)
    
    exportAnimFile(saveDir, animBonesList, settings, doEditSettings)
    
#Actual export for animations
def exportAnimFile(saveDir, bones, settingsFile, doEditSettings):
    
    print settingsFile
    
    cmds.GrannyLoadSettings(mainPath + settingsFile)
    
    saveDir += ".lsm"
    

    for bone in bones:
        cmds.select(bone, add=True)
    
    didCheckout = False 
    saveDir = saveDir.replace('\\', '/')
   
    if MLEUtils.canWritePath(saveDir):   
        cmds.GrannyExport(saveDir, s = True, i=doEditSettings, ca=True)
    
        