import maya.cmds as cmds
import maya.mel as mel

#python imports
import sys
import re
import os

from . import MLEUtils

mainPath = os.environ['MAYA_APP_DIR']+ "/scripts/LarianStudiosModTools/" #Best guess for maya path install

#presets
granSetMesh = "Gr2_Presets\Mesh_ExportSettings_Maya.ges"
granSetVertCols = "Gr2_Presets\Mesh_VertexClrs_ExportSettings_Maya.ges"
granSetProxyMesh = "Gr2_Presets\MeshProxy_ExportSettings_Maya.ges"



#export RegEx
physCheck =  re.compile(".*_PHYS",re.IGNORECASE)
boneCheck =  re.compile(".*_BONE$|.*BONE_.*",re.IGNORECASE)
nonConstrBoneCheck = re.compile(".*_BONE$|.*BONE_.*",re.IGNORECASE)

#get develop path and throw errors when the file isn't found or the file is empty or misisng the client root entry
def GrannyMeshExport(sourceFile, targetPath, selectedLayers, doVisual, doVertexColor, doIncludeBones, doEditSettings, doExpressExport):
            
    
    #load relevant plugins
    cmds.loadPlugin("larian_maya2015_x64.mll")
    
    #open the file if in dos mode
    if not doExpressExport:
        cmds.file(sourceFile, open=True, force=True)
               
    baseLayer = None        
    layers = cmds.ls(long=True,type='displayLayer')
         
    for lay in layers:
        if "Base" in lay:
            baseLayer = lay
        if lay.find("defaultLayer") == -1: #default layer can't be set to invisible
            cmds.setAttr(lay+".visibility", 0)                
                
    #cycle layer show one by one and export content according to checks
    for layer in selectedLayers:       
        try:               
            visualsList = []
            isCharacter = False
                
            cmds.setAttr(layer+".visibility", 1)
            cmds.select(clear=True)
                
            layerContents = cmds.editDisplayLayerMembers(layer, q=True, fullNames=True)#using full names due to reoccuring selection errors with bones            
                          
            for content in layerContents:
                if not physCheck.match(content) and not cmds.objectType(content) =='bulletRigidBodyShape': #ignore bones and bulletshapes and phys objects
                    visualsList.append(content)
                
            print "INCLUDE BONES"
            print doIncludeBones
            
            print visualsList
            bonesList = []
            root = None                
            
            SkinBoneList = []
            
            for visual in visualsList:
                skinCluster = mel.eval('findRelatedSkinCluster '+ visual)
                
                if len(skinCluster) > 0:
                    SkinBoneList = SkinBoneList + cmds.skinCluster(skinCluster,query=True,inf=True)
                    isCharacter = True
                    break
            
            print "skin parts"
            if isCharacter or 'Base' in layer:
                if isCharacter:
                    root= SkinBoneList[0]
                else:
                    root = layerContents[0]
                
                while (True):
                    parent = cmds.listRelatives( root, parent=True, type='joint' )
                    if not parent:
                        break;

                    root = parent[0]
            
            print "THE ROOT"
            print root    
            print isCharacter  
                           
            if not doIncludeBones and isCharacter: 
                if cmds.attributeQuery('MLEData', node=root ,exists=True):
                    print cmds.getAttr(root+'.MLEData')
                    if cmds.getAttr(root+'.MLEData') in  "Single = true":
                        doIncludeBones=True
                
            print doIncludeBones
                
            #If the layer is a base layer get all bones if single = true get all bones else get bones from skin                           
            if isCharacter or ("Base" in layer or doIncludeBones):
                if doIncludeBones or "_Base" in layer:
                    print "Getting all bones"
                    #make this get the dummy root and select hierarchy
                    cmds.select(root, hierarchy=True)
                    for bones in cmds.ls(selection=True):
                        bonesList.append(bones)
                else:
                    for visual in visualsList:
                        skin = mel.eval('findRelatedSkinCluster ' + visual)
                            
                        if len(skin)>0:
                            skinBones = cmds.skinCluster(skin, q=True, inf=True)
                            print skinBones
                            
                            for bone in skinBones:
                                bonesList.append(bone)  
                cmds.select(bonesList)
                            
            VertexCheck = False
            ProxyCheck  = False  

            #add all the visuals to the selection check for vertex color export
            for visual in visualsList:
                cmds.select(visual,add=True)
                if cmds.attributeQuery('UserDefinedProperties', node=visual, exists=True):
                    clothVar = cmds.getAttr(visual+'.UserDefinedProperties')
                    if clothVar is None:
                        clothVar = "Empty"
                    if clothVar in 'Cloth=1'or clothVar in 'VertexColoured':
                        VertexCheck = True
                    if clothVar in 'MeshProxy':
                        ProxyCheck = True

            #check if vertex color needs to be included load settings accordingly     
            if doVisual:
                if doVertexColor or VertexCheck or ProxyCheck:
                    if ProxyCheck:
                        print "using proxy mesh settings"
                        cmds.GrannyLoadSettings(mainPath+granSetProxyMesh)
                    else:
                        print "using vertex settings"
                        cmds.GrannyLoadSettings(mainPath+granSetVertCols)
                else:
                    print "using regular settings"
                    cmds.GrannyLoadSettings(mainPath+granSetMesh)
                    
               #go through selection and count UV sets set Coords per vertex to counted number with GrannySetValue with s = true
                currentlySelectedGeo = cmds.ls(selection=True)
                for geo in currentlySelectedGeo: 
                    cmds.select(geo)
                    if not cmds.polyUVSet(q=True, allUVSets=True) is None:
                        cmds.GrannySetValue("Coords per vertex", len(cmds.polyUVSet(q=True, allUVSets=True)), "mesh", geo, s=True)
                
                print "past uv check"    
                cmds.select(currentlySelectedGeo)
                            
                #Check geometry scale  
                for geo in cmds.ls(selection=True):
                    MLEUtils.checkScale(geo)
                
                print "past scale check" 
                        
                saveDir = targetPath + layer + ".lsm"
                    
                print saveDir
                    
                print cmds.ls(selection=True)
                       

                if MLEUtils.canWritePath(targetPath):    
                    cmds.GrannyExport(saveDir, s = True, i=doEditSettings, ca=True)
                    
                    
            cmds.setAttr(layer+".visibility", 0)
            cmds.select(clear=True)
                
            output = ""
        except Exception, e:
            print e
                
            

        
        