import maya.mel as mel
import maya.cmds as cmds

import os
import subprocess

def getDevelopPath():
    projectDirectory = cmds.workspace(q=True, rd=True)
    scriptPathList = os.environ['MAYA_SCRIPT_PATH'].split(";")
    #get the path for history saving
    for path in scriptPathList:
        if projectDirectory in path:
            defaultScriptPath = path
            
    return defaultScriptPath
    
    
def canWritePath(path):
    return os.path.exists(path[:path.rfind('/')]) and (os.access(path, os.W_OK) or not os.path.exists(path))

#Check if the objects beign exported have a scale and warn the user
def checkScale(geometry):
    
    attrs = ['sx', 'sy', 'sz']
    
    if not cmds.objectType(geometry) == 'joint' and not 'Dummy' in geometry:
        objScale = [cmds.getAttr(geometry+'.'+attr) for attr in attrs]
        
        if not objScale == [1.0,1.0,1.0]:
            return False
def startDebugLog():
    debugFile = open(getDevelopPath()+"MLEDebugLog.txt", 'w')
    
    debugFile.close()
    
    
def writeToDebugLog(text):
    debugFile = open(getDevelopPath()+"MLEDebugLog.txt", 'a')
    
    debugFile.write(text + '\r\n')
    
    debugFile.close()
    
def showDebugFile():
    path= getDevelopPath()+"MLEDebugLog.txt"
    
    subprocess.Popen('explorer '+path.replace('/', '\\'))

#Check if this is an animation file
def isAnimFile():
    sceneName = cmds.file(q=True, sceneName=True, shortName=True)
    checkName = cmds.file(q=True, sceneName=True)
    
    checkName = checkName[:len(checkName)-len(sceneName)-1]+".ma"
        
    return cmds.file(checkName,q=True, ex=True)
    
#Check if this is an animation base file
def isAnimBaseFile():
    checkFolder = cmds.file(q=True, sceneName=True)[:-3]
    
    return os.path.isdir(checkFolder)

#get animation manager data if available from a dummy root if it's found    
def getAnimationList():
    
    global fullAnimList
    
    fullAnimList=[]
    
    if isAnimBaseFile():
        fullAnimList = cmds.getFileList(folder=cmds.file(q=True, sceneName=True)[:-3], filespec="*.ma")
        return fullAnimList
    else:
        
        if isAnimFile():
            global dummyRoot
            dummyRoot = getRootDummy()
            print dummyRoot
            if dummyRoot == None:
                return ["No root dummy found"]
            else:
                for root in dummyRoot:
                    if cmds.attributeQuery('AmData', node=root ,exists=True):
                        dataString = cmds.getAttr(root+'.AmData')
                        if not dataString is None:
                            splitList = dataString.split(',')
                            
                            print root
                            print splitList

                             
                            for i in xrange(len(splitList)):
                                
                                if not splitList[i] is None and not splitList[i] == '':
                                   fullAnimList.append(splitList[i])
                print fullAnimList
                           
                if len(fullAnimList) == 0:
                    return ["No animation manager data found"]
            
                
                return fullAnimList
        else:
            return ["No animation folder found"]
    
#get the root dummy if available
def getRootDummy():
    dummyList=[]
    sceneObjects = cmds.ls()
    for obj in sceneObjects:
        if cmds.objectType(obj) == 'joint':
             if 'Dummy_' in obj and not '|' in obj:
                 dummyList.append(obj)
        
    if dummyList == None or len(dummyList) == 0:
        return None
    else:
        return dummyList







