#!python2

import os
import sys

from functools import partial

import maya.cmds
import maya.mel
import maya.utils


BASE_EXPORT_DIR = 'C:/Art'


def init(selfdir):
    selfdir = os.path.abspath(selfdir)
    maya.utils.executeDeferred(partial(_create_shelf, _SHELF_NAME, selfdir))


def export_to_json(selfdir):
    _export_all_layers(selfdir, 'json')


def export_to_bullet(selfdir):
    _export_all_layers(selfdir, 'bullet')


def _export_all_layers(selfdir, output_type):
    sys.path.append(selfdir)
    sys.path.append(os.path.join(selfdir, '../../python'))

    import larian_paths  # noqa
    reload(larian_paths)  # noqa

    import bullet_exporter
    reload(bullet_exporter)  # noqa

    for layer_name in maya.cmds.ls(type='displayLayer'):
        if layer_name.endswith('Ragdoll'):
            path = _get_export_path(layer_name, output_type)

            exporter = getattr(bullet_exporter, 'export_scene_to_' + output_type)

            exporter(layer_name, path)


def _get_export_path(layer_name, output_type):
    name = layer_name + '.' + output_type
    return os.path.join(BASE_EXPORT_DIR, name)


_BUTTONS = {
    "JSON": export_to_json,
    "Bullet": export_to_bullet
}

_MODULE_NAME = os.path.basename(__name__)

_SHELF_NAME = "ButterBulletExporterTest"


def _create_shelf(shelf_name, selfdir):
    parent_shelf = maya.mel.eval('global string $gShelfTopLevel; $tempVar = $gShelfTopLevel;')

    shelf_already_exists = maya.cmds.shelfLayout(shelf_name, exists=True)

    if shelf_already_exists:
        buttons = maya.cmds.shelfLayout(shelf_name, q=True, childArray=True) or []

        for button in buttons:
            maya.cmds.deleteUI(button, control=True)
    else:
        shelf = maya.cmds.shelfLayout(shelf_name, cellWidth=33, cellHeight=33, parent=parent_shelf)

    for name, function in _BUTTONS.iteritems():
        command = 'import imp; imp.load_source("{}", "{}/{}.py").{}("{}")'.format(
            _MODULE_NAME, selfdir, _MODULE_NAME, function.__name__, selfdir
        )

        maya.cmds.shelfButton(command=command,
                              label=name,
                              parent=shelf_name,
                              enable=True,
                              manage=True,
                              visible=True,
                              style="iconOnly",
                              image="vacantCell.png",
                              imageOverlayLabel=name)

    shelves = maya.cmds.shelfTabLayout(parent_shelf, query=True, tabLabelIndex=True)

    for i, shelf in enumerate(shelves):
        maya.cmds.optionVar(stringValue=('shelfName{}'.format(i + 1), str(shelf)))
