import glob
import logging
import os
import subprocess


def write_export_data(data, path, output_type_string):
    self_dir = os.path.dirname(__file__)

    processor_path = os.path.join(self_dir, 'larian_physics_processor.exe')

    if os.path.exists(processor_path):
        cmd_base = [processor_path]
    else:
        processor_path = os.path.join(self_dir, 'larian_physics_processor.py')
        cmd_base = [_find_python3_interpreter(), processor_path]

    cmd = cmd_base + ['-i', '-', '--input-type', 'json',
                      '-o', path, '--output-type', output_type_string,
                      '--coordinate-system', 'right_up_backward',
                      '--log-pipe-format']

    processor = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = processor.communicate(input=data)

    for line in out.splitlines():
        print(line)

    success = processor.returncode == 0

    messages = [(logging.ERROR, [])]

    for line in err.splitlines():
        if line.startswith('!log:'):
            _, levelno, message = line.split(':', 2)

            messages.append((int(levelno), [message]))
        else:
            messages[-1][1].append(line)

    for levelno, lines in messages:
        if len(lines) > 0:
            message = "\n\t".join(lines)
            logging.log(levelno, message)

    return success


def _find_python3_interpreter():
    for prefix in os.environ['PATH'].split(os.pathsep):
        if len(glob.glob(os.path.join(prefix, 'python3*.dll'))) > 0:
            return os.path.join(prefix, 'pythonw.exe')

    return None
